import datetime
import pytest
import mock
from copy import deepcopy
import os
import json
import shutil
import tempfile
import time
import botocore.auth
import iotbotocredentialprovider.AWS
import iotbotocredentialprovider.FakeMetadata


metadata = {
    'account_id': '0123456789',
    'certificate_id': 'mycertificateid',
    'credential_endpoint': 'https://xyzzy.credentials.iot.us-east-1.amazonaws.com',
    'device_name': 'test1',
    'region': 'us-test-1',
    'role_alias_name': 'TestRole'
}

fake_credentials = {
    'accessKeyId': 'MyAccessKey',
    'expiration': '2018-03-12T03:52:05Z',
    'secretAccessKey': 'MySecretAccessKey',
    'sessionToken': 'MySessionToken',
}


class TestFakeMetadata(object):
    def setup(self):
        self.registration_dir = tempfile.mkdtemp()
        self.metadata_file = os.path.join(self.registration_dir, "metadata.json")

        with open(self.metadata_file, "w") as f:
            json.dump(metadata, f)

        self.cp = iotbotocredentialprovider.FakeMetadata.FakeMetadataCredentialProvider(self.registration_dir)
        assert self.cp.path == self.registration_dir

    def teardown(self):
        shutil.rmtree(self.registration_dir)


    def test_metadata_credentials(self):
        expire_time = datetime.datetime.utcnow() + datetime.timedelta(hours=1)
        self.cp._credentials = fake_credentials
        self.cp._credential_expiration = expire_time

        metadata_creds = self.cp.metadata_credentials
        assert metadata_creds == {
            'AccessKeyId': fake_credentials['accessKeyId'],
            'SecretAccessKey': fake_credentials['secretAccessKey'],
            'Token': fake_credentials['sessionToken'],
            'Expiration': fake_credentials['expiration']
        }

    def test_role_name(self):
        assert self.cp.role_name == metadata['role_alias_name']

    @mock.patch.object(iotbotocredentialprovider.FakeMetadata.FakeMetadataCredentialProvider, "get_credentials")
    def test_update_timer(self, mock_get_credentials):
        self.cp.update_timer(refresh_time_seconds=1)
        time.sleep(2)
        assert mock_get_credentials.called is True

    def test_cancel_timer_no_timer(self):
        assert not hasattr(self.cp, "_update_timer")
        self.cp.cancel_timer()
        assert not hasattr(self.cp, "_update_timer")

    @mock.patch.object(iotbotocredentialprovider.FakeMetadata.FakeMetadataCredentialProvider, "get_credentials")
    def test_cancel_timer(self, mock_get_credentials):
        self.cp.update_timer(refresh_time_seconds=2)
        time.sleep(1)
        self.cp.cancel_timer()
        time.sleep(2)
        assert mock_get_credentials.called is False

    @mock.patch.object(iotbotocredentialprovider.FakeMetadata.FakeMetadataCredentialProvider, "get_credentials")
    def test_get_refresh_seconds(self, mock_get_credentials):
        mock_get_credentials.return_value = deepcopy(fake_credentials)
        expire_time = datetime.datetime.utcnow() + datetime.timedelta(hours=1)
        mock_get_credentials.return_value['expiration'] = expire_time.strftime(botocore.auth.ISO8601)

        refresh = self.cp.get_refresh_seconds()
        assert refresh > 0.7*3600
        assert refresh < 3600



import pytest
import datetime
import tempfile
import time
from copy import deepcopy
import json
import mock
import os
import botocore
import botocore.session
import shutil
import boto3
import requests
import iotbotocredentialprovider.AWS
from botocore.credentials import CredentialProvider, RefreshableCredentials
import requests.packages.urllib3.util.connection as urllib3_cn


metadata = {
    'account_id': '0123456789',
    'certificate_id': 'mycertificateid',
    'credential_endpoint': 'https://xyzzy.credentials.iot.us-east-1.amazonaws.com',
    'device_name': 'test1',
    'region': 'us-test-1',
    'role_alias_name': 'TestRole'
}

fake_credentials = {
    'accessKeyId': 'MyAccessKey',
    'expiration': '2018-03-12T03:52:05Z',
    'secretAccessKey': 'MySecretAccessKey',
    'sessionToken': 'MySessionToken',
}


class TestIotBotoCredentialProvider(object):
    def setup(self):
        self.registration_dir = tempfile.mkdtemp()
        self.metadata_file = os.path.join(self.registration_dir, "metadata.json")

        with open(self.metadata_file, "w") as f:
            json.dump(metadata, f)

        self.cp = iotbotocredentialprovider.AWS.IotBotoCredentialProvider(self.registration_dir)
        assert self.cp.path == self.registration_dir

    def teardown(self):
        shutil.rmtree(self.registration_dir)

    def test_iot_metadata_path(self):
        cp = iotbotocredentialprovider.AWS.IotBotoCredentialProvider()
        assert cp.path == iotbotocredentialprovider.AWS.default_iot_metadata_path

    def test_alternate_iot_metadata_path(self):
        iot_metadata_path = "/foo"
        cp = iotbotocredentialprovider.AWS.IotBotoCredentialProvider(iot_metadata_path)
        assert cp.path == iot_metadata_path

    def test_metadata(self):
        assert not hasattr(self.cp, "_metadata")
        md = self.cp.metadata
        assert hasattr(self.cp, "_metadata")
        assert self.cp._metadata == md
        assert md == metadata

    def test_credentials_cached(self):
        expire_time = datetime.datetime.utcnow() + datetime.timedelta(hours=1)
        self.cp._credentials = fake_credentials
        self.cp._credential_expiration = expire_time
        assert self.cp.credentials == fake_credentials

    @mock.patch.object(iotbotocredentialprovider.AWS.IotBotoCredentialProvider, "get_credentials")
    def test_credentials_expired(self, mock_get_credentials):
        expire_time = datetime.datetime.utcnow() - datetime.timedelta(hours=1)
        self.cp._credentials = fake_credentials
        self.cp._credential_expiration = expire_time

        mock_get_credentials.return_value = {}
        assert self.cp.credentials == {}
        assert mock_get_credentials.called is True

    @mock.patch.object(iotbotocredentialprovider.AWS.IotBotoCredentialProvider, "get_credentials")
    def test_refresh_credentials(self, mock_get_credentials):
        self.cp._credentials = "test"
        mock_get_credentials.return_value = fake_credentials
        self.cp._refresh_credentials()
        assert mock_get_credentials.called is True
        assert not hasattr(self.cp, "_credentials")

    @mock.patch.object(iotbotocredentialprovider.AWS.IotBotoCredentialProvider, "get_credentials")
    def test_fetch_metadata(self, mock_get_credentials):
        mock_get_credentials.return_value = fake_credentials
        boto3_creds = self.cp._fetch_metadata()
        assert mock_get_credentials.called is True
        assert boto3_creds == {
            'access_key': fake_credentials['accessKeyId'],
            'secret_key': fake_credentials['secretAccessKey'],
            'token': fake_credentials['sessionToken'],
            'expiry_time': fake_credentials['expiration']
        }

    @mock.patch.object(iotbotocredentialprovider.AWS.IotBotoCredentialProvider, "get_credentials")
    def test_load(self, mock_get_credentials):
        mock_get_credentials.return_value = fake_credentials
        res = self.cp.load()
        assert mock_get_credentials.called is True
        assert isinstance(res, RefreshableCredentials)

    @mock.patch.object(iotbotocredentialprovider.AWS.IotBotoCredentialProvider, "_fetch_metadata")
    def test_load_no_creds(self, mock_fetch_metadata):
        mock_fetch_metadata.return_value = {}
        res = self.cp.load()
        assert mock_fetch_metadata.called is True
        assert res is None

    def test_boto3_credentials(self):
        expire_time = datetime.datetime.utcnow() + datetime.timedelta(hours=1)
        self.cp._credentials = fake_credentials
        self.cp._credential_expiration = expire_time

        boto3_creds = self.cp.boto3_credentials
        assert boto3_creds == {
            'access_key': fake_credentials['accessKeyId'],
            'secret_key': fake_credentials['secretAccessKey'],
            'token': fake_credentials['sessionToken'],
            'expiry_time': fake_credentials['expiration']
        }

    def test_get_botocore_session(self):
        bc = self.cp.get_botocore_session()
        assert isinstance(bc, botocore.session.Session)

    def test_get_boto3_session(self):
        bs = self.cp.get_boto3_session("us-east-1")
        assert isinstance(bs, boto3.session.Session)

    @mock.patch("requests.get")
    def test_get_credentials_200(self, mock_requests_get):
        response = mock.Mock()
        response.status_code = 200
        response_data = {'credentials': fake_credentials}
        response.text = json.dumps(response_data)
        mock_requests_get.return_value = response

        res = self.cp.get_credentials()
        assert res == fake_credentials

    @mock.patch("requests.get")
    def test_get_credentials_403(self, mock_requests_get):
        response = mock.Mock()
        response.status_code = 403
        response_data = '{"message":"Access Denied"}'
        response.text = json.dumps(response_data)
        mock_requests_get.return_value = response

        with pytest.raises(iotbotocredentialprovider.AWS.IotBotoCredentialProviderError):
            self.cp.get_credentials()


class TestGetSessions(object):
    def setup(self):
        self.registration_dir = tempfile.mkdtemp()
        self.metadata_file = os.path.join(self.registration_dir, "metadata.json")

        with open(self.metadata_file, "w") as f:
            json.dump(metadata, f)

    def teardown(self):
        shutil.rmtree(self.registration_dir)

    def test_get_botocore_session(self):
        bc = iotbotocredentialprovider.AWS.get_botocore_session(iot_metadata_path=self.registration_dir)
        assert isinstance(bc, botocore.session.Session)

    def test_get_boto3_session(self):
        bs = iotbotocredentialprovider.AWS.get_boto3_session("us-east-1",
                                                             iot_metadata_path=self.registration_dir)
        assert isinstance(bs, boto3.session.Session)

    def test_configure_session(self):
        my_session = botocore.session.Session()
        original_component_list = set(deepcopy(my_session.get_component('credential_provider').providers))
        for x in original_component_list:
            assert not isinstance(x, iotbotocredentialprovider.AWS.IotBotoCredentialProvider)

        botocore_session = iotbotocredentialprovider.AWS.configure_session(iot_metadata_path=self.registration_dir,
                                                                           session=my_session)

        new_component_list = set(botocore_session.get_component('credential_provider').providers)

        saw_cred_provider = False
        for x in new_component_list:
            if isinstance(x, iotbotocredentialprovider.AWS.IotBotoCredentialProvider):
                saw_cred_provider = True
        assert saw_cred_provider is True



import botocore.auth
import platform
import datetime
import json
import logging
import os
import random
from threading import Timer
from .AWS import IotBotoCredentialProvider, default_iot_metadata_path

try:
    from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
except ImportError:
    from http.server import BaseHTTPRequestHandler, HTTPServer


log = logging.getLogger()
log.setLevel(logging.INFO)

# hosts = ["169.254.169.254", "169.254.170.2"]
# loopback is preferred, this will require that
# the following be set:
# /sbin/iptables -t nat -A OUTPUT -p tcp -d 169.254.169.254 --dport 80 -j DNAT --to-destination 127.0.0.1:51680
# /sbin/iptables -t nat -A OUTPUT -p tcp -d 169.254.170.2   --dport 80 -j DNAT --to-destination 127.0.0.1:51680

HOST = "127.0.0.1"
PORT = 51680
HOST = "0.0.0.0"
ROLE_PATH = "/latest/meta-data/iam/security-credentials"
IDENTITY_PATH = "/latest/dynamic/instance-identity/document"
SIGNATURE_PATH = "/latest/dynamic/instance-identity/signature"
PLACEMENT_AVAILABILITY_ZONE_PATH = "/latest/meta-data/placement/availability-zone"
PING_PATH = "/ping"
PING_RESPONSE = "pong"
INSTANCE_DOCUMENT_OVERRIDE_FILE = os.path.join(default_iot_metadata_path, "instance_document_overrides.json")


def json_serial(obj):
    """
    JSON serializer for objects not serializable by default json code
    e.g. datetime.datetime objects
    """

    if isinstance(obj, (datetime.datetime, datetime.date)):
        return obj.isoformat()
    raise TypeError("Type %s not serializable" % type(obj))


class FakeMetadataCredentialProvider(IotBotoCredentialProvider):
    @property
    def role_name(self):
        return self.metadata['role_alias_name']

    @property
    def metadata_credentials(self):
        return {
            'AccessKeyId': self.credentials['accessKeyId'],
            'SecretAccessKey': self.credentials['secretAccessKey'],
            'Token': self.credentials['sessionToken'],
            'Expiration': self.credentials['expiration'],
            'Code': 'Success',
            'Type': 'AWS-HMAC',
            'LastUpdated': self.credentials['expiration']
        }

    @property
    def account(self):
        return self.metadata["account_id"]

    @property
    def region(self):
        return self.metadata["region"]

    def update_timer(self, refresh_time_seconds=300):
        self._update_timer = Timer(refresh_time_seconds, self.get_credentials)
        self._update_timer.daemon = True
        logging.info("will refresh creds in %s", refresh_time_seconds)
        self._update_timer.start()

    def cancel_timer(self):
        if hasattr(self, "_update_timer"):
            self._update_timer.cancel()

    def get_refresh_seconds(self):
        if not hasattr(self, "_credential_expiration"):
            expire_time = datetime.datetime.strptime(self.credentials['expiration'],
                                                     botocore.auth.ISO8601)
            self._credential_expiration = expire_time

        now = datetime.datetime.utcnow()
        expiration = (self._credential_expiration - now).seconds
        logging.debug("credentials expire in %s seconds", expiration)
        refresh_jitter = int(0.1 * expiration)
        if refresh_jitter < 30:
            refresh_jitter = 30
        refresh_time = 0.7 * expiration + random.randrange(0, refresh_jitter)
        return refresh_time

    def get_credentials(self):
        result = super(FakeMetadataCredentialProvider, self).get_credentials()
        self.update_timer(self.get_refresh_seconds())
        return result


class FakeMetadataRequestHandler(BaseHTTPRequestHandler):
    """
    This implements the request handling that we'll
    need, it responds very simply:

    if user requests ROLE_PATH, respond with the role name we serve
    if user requests ROLE_PATH + role name, respond with credentials
        obtained by self.get_credentials(RoleArn)
    otherwise, return a 404

    This class shouldn't directly be used, instead use a child
    which implements get_credentials

    """
    # we want to use the same provider across all class instances
    # to allow for caching
    credential_provider = FakeMetadataCredentialProvider()

    def get_credentials(self, RoleArn=None):
        return FakeMetadataRequestHandler.credential_provider.metadata_credentials

    def get_role(self):
        return FakeMetadataRequestHandler.credential_provider.role_name

    def get_placement_availability_zone(self):
        result = "fake"
        try:
            override = json.load(open(INSTANCE_DOCUMENT_OVERRIDE_FILE))
            result = override.get("availabilityZone", result)
        except (ValueError, IOError):
            pass

        return result

    def get_identity_doc(self):
        result = {
            "accountId": FakeMetadataRequestHandler.credential_provider.account,
            "region": FakeMetadataRequestHandler.credential_provider.region,
            "architecture": platform.machine(),
            "availabilityZone": "fake",
            "imageId": "fake",
            "instanceId": FakeMetadataRequestHandler.credential_provider.metadata['device_name'],
            "instanceType": "f1.fake",
            "privateIp": "fake",
        }

        try:
            override = json.load(open(INSTANCE_DOCUMENT_OVERRIDE_FILE))
            result.update(override)
        except (ValueError, IOError):
            pass

        return result

    def do_GET(self):
        our_role = self.get_role()
        our_path = ROLE_PATH + "/" + self.get_role()
        return_code = 200
        start_doc = "HTTP/1.0 200 OK\Content-Type: text/plain\n\n"
        result = ""

        stripped_path = self.path.rstrip("/")
        if stripped_path == PING_PATH:
            result = PING_RESPONSE
        elif stripped_path == ROLE_PATH:
            # client is requesting we return the role name
            result = our_role
        elif stripped_path == PLACEMENT_AVAILABILITY_ZONE_PATH:
            result = self.get_placement_availability_zone()
        elif stripped_path == IDENTITY_PATH:
            result = json.dumps(self.get_identity_doc(), default=json_serial, indent=4)
        elif stripped_path == SIGNATURE_PATH:
            result = "bad"
        elif stripped_path != our_path:
            # client asked for a role we don't serve
            return_code = 404
            start_doc = "HTTP/1.0 400 Bad Request\nContent-Type: text/html\n"
            result = """
<?xml version="1.0" encoding="iso-8859-1"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
         "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
 <head>
  <title>404 - Not Found</title>
 </head>
 <body>
  <h1>404 - Not Found</h1>
 </body>
</html>
"""
        else:
            # client asked for credentials
            # start_doc ="HTTP/1.0 200 OK\nContent-Type: application/json\n\n"
            credentials = self.get_credentials()
            result = json.dumps(credentials, default=json_serial, indent=4)
        self.send_response(return_code)
        self.wfile.write(bytes(start_doc.encode("utf-8") + result.encode("utf-8")))


class FakeMetadataServer(object):
    """
    This creates a server which acts like METADATA
    You will need to have certain traffic directed to it, e.g.

    /sbin/sysctl -w net.ipv4.conf.all.route_localnet=1
    /sbin/iptables -t nat -A PREROUTING -p tcp -d 169.254.169.254 --dport 80 -j DNAT --to-destination 127.0.0.1:51679

    TBD: may not need to redirect the container address, this may
    be best left to ECSAgent, the container address is:

    /sbin/iptables -t nat -A PREROUTING -p tcp -d 169.254.170.2   --dport 80 -j DNAT --to-destination 127.0.0.1:51679

    """

    def __init__(self, request_handler, host=None, port=None):
        self.request_handler = request_handler
        if host is None:
            self.host = HOST
        else:
            self.host = host

        self.port = port
        if self.port is None:
            self.port = PORT

        self.server = HTTPServer((self.host, self.port), self.request_handler)

    def stop(self):
        self.request_handler.credential_provider.cancel_timer()
        self.server.shutdown()
        self.server.server_close()

    def run(self):
        print("run server on %s:%s" % (self.host, self.port))
        self.server.serve_forever()
        self.request_handler.credential_provider.cancel_timer()
        self.server.shutdown()
        self.server.server_close()



import boto3
import datetime
import requests
import json
import os
import logging
import botocore.auth
from botocore.credentials import CredentialProvider, RefreshableCredentials


log = logging.getLogger()
log.setLevel(logging.INFO)

default_iot_metadata_path = "/AWSIoT"


class IotBotoCredentialProviderError(Exception):
    pass


class IotBotoCredentialProvider(CredentialProvider):
    def __init__(self, iot_metadata_path=default_iot_metadata_path):
        self.path = iot_metadata_path
        self._metadata_file = os.path.join(self.path, "metadata.json")

    @property
    def metadata(self):
        if not hasattr(self, "_metadata") or \
                os.stat(self._metadata_file) != self._metadata_mtime:

            self._populate_metadata()
        return self._metadata

    def _populate_metadata(self):
        with open(self._metadata_file) as f:
            self._metadata = json.load(f)
            self._metadata_mtime = os.stat(self._metadata_file).st_mtime

    @property
    def credentials(self):
        now = datetime.datetime.utcnow()

        if hasattr(self, "_credentials") and \
            hasattr(self, "_credential_expiration") and \
                self._credential_expiration > now:

            return self._credentials

        return self.get_credentials()

    def get_credentials(self):
        url = "%s/role-aliases/%s/credentials" % (self.metadata['credential_endpoint'],
                                                  self.metadata['role_alias_name'])

        headers = {"x-amzn-iot-thingname": self.metadata['device_name']}

        certificate_file = os.path.join(self.path, "%s.pem" % self.metadata['certificate_id'])
        private_key_file = os.path.join(self.path, "%s.privatekey" % self.metadata['certificate_id'])

        o = requests.get(url, cert=(certificate_file, private_key_file), headers=headers)
        response = json.loads(o.text)

        if o.status_code == 200:
            self._credentials = response["credentials"]
            self._credential_expiration = datetime.datetime.strptime(self._credentials['expiration'],
                                                                     botocore.auth.ISO8601)
            return self._credentials

        raise IotBotoCredentialProviderError(response)

    @property
    def boto3_credentials(self):
        return {
            'access_key': self.credentials['accessKeyId'],
            'secret_key': self.credentials['secretAccessKey'],
            'token': self.credentials['sessionToken'],
            'expiry_time': self.credentials['expiration']
        }

    def _refresh_credentials(self):
        if hasattr(self, "_credentials"):
            del self._credentials
        return self.credentials

    def _fetch_metadata(self):
        self._refresh_credentials()
        return self.boto3_credentials

    def load(self):
        fetcher = self._fetch_metadata

        metadata = fetcher()
        if not metadata:
            return None

        log.debug("Obtained for account %s will expire at %s",
                  self.metadata['account_id'], self.credentials['expiration'])

        return RefreshableCredentials.create_from_metadata(
            metadata,
            method=self.METHOD,
            refresh_using=fetcher,
        )

    def get_botocore_session(self, insert_before='iam-role'):
        session = botocore.session.Session()
        session.get_component('credential_provider').insert_before(insert_before, self)
        return session

    def get_boto3_session(self, region_name, insert_before='iam-role'):
        botocore_session = self.get_botocore_session(insert_before=insert_before)
        boto3_session = boto3.session.Session(
            botocore_session=botocore_session,
            region_name=region_name
        )
        return boto3_session


def configure_session(session, iot_metadata_path=default_iot_metadata_path, insert_before='iam-role'):
    """Configure a Botocore session to obtain credentials from AWS IoT.

    :param session: Existing botocore Session
    :type session: :class:`botocore.session.Session`
]   :param str iot_metadata_path: where to look for AWS IoT registration files (metadata.json,
            and certificates)
    :returns: Botocore session with auto-updating AWS IoT federated credentials
    :rtype: :class:`botocore.session.Session`

    .. code-block:: python

        import botocore
        import botocore.session
        import boto3

        region_name = 'us-east-1'
        account_id = '123456789'

        botocore_session = iotbotocredentialprovider.AWS.configure_session(
            session=botocore.session.Session(),
            account_id=account_id)

        # If application uses the default session
        boto3.setup_default_session(
            botocore_session=botocore_session,
            region_name=region_name
        )
        # If application needs individual sessions
        boto3_session = boto3.session.Session(
            botocore_session=botocore_session,
            region_name=region_name
        )


    """
    # we choose to configure our credentials before IAM
    session.get_component('credential_provider').insert_before(
        insert_before, IotBotoCredentialProvider(iot_metadata_path=iot_metadata_path))
    return session


def get_botocore_session(iot_metadata_path=default_iot_metadata_path, insert_before='iam-role'):
    """
    :param str iot_metadata_path=default_iot_metadata_path: The path to the IoT registration directory
        which includes metadata.json, the certificate, and its private key
    :param str insert_before: where in the list to insert the material set

    Obtain a botocore session using a iotbotocredentialprovider credential provider to renew credentials.

    """
    cp = IotBotoCredentialProvider(iot_metadata_path)
    return cp.get_botocore_session(insert_before)


def get_boto3_session(region_name, iot_metadata_path=default_iot_metadata_path, insert_before='iam-role'):
    """
    :param str iot_metadata_path=default_iot_metadata_path: The path to the IoT registration directory
        which includes metadata.json, the certificate, and its private key
    :param str region_name: aws region name, e.g. us-east-1, us-west-2, etc
    :param str insert_before: where in the list to insert the material set

    Obtain a boto3 session using a iotbotocredentialprovider credential provider to renew credentials.
    """
    cp = IotBotoCredentialProvider(iot_metadata_path=iot_metadata_path)
    return cp.get_boto3_session(region_name, insert_before)



#!/usr/bin/env python3
import argparse
from iotbotocredentialprovider.FakeMetadata import FakeMetadataServer, FakeMetadataRequestHandler, PORT

# this will require that
# the following be set:
# /sbin/iptables -t nat -A OUTPUT -p tcp -d 169.254.169.254 --dport 80 -j DNAT --to-destination 127.0.0.1:51680
# /sbin/iptables -t nat -A OUTPUT -p tcp -d 169.254.170.2   --dport 80 -j DNAT --to-destination 127.0.0.1:51680

if __name__ == "__main__":
    parser = argparse.ArgumentParser()
    parser.add_argument("--port", type=int, dest="port",
                        help="port to listen on defaults to %s" % PORT, required=False, default=PORT)
    args = parser.parse_args()

    f = FakeMetadataServer(FakeMetadataRequestHandler, port=args.port)
    f.run()



#!/usr/bin/env python

import os
from setuptools import setup, find_packages

setup(name='iotbotocredentialprovider',
      version='1.0.4',
      description='AWS IoT Credential Provider: create boto sessions which obtain and renew credentials from an AWS IoT device certificate',
      author='Craig I. Hagan',
      author_email='hagan@cih.com',
      url='https://github.com/craighagan/iotbotocredentialprovider',
      license='MIT',
      packages = find_packages(exclude=["test"]),
      install_requires=["boto3","requests"],
      setup_requires=["pytest-runner"],
      tests_require=["pytest", "pytest-runner"],
      scripts=["bin/fakemetadata-server.py"],
)
